﻿/*---------------------------------------------------------------------------*
# Colorbit Sample
 *---------------------------------------------------------------------------*/

#include <nn.h>
#include <nn/types.h>
#include <nn/dbg.h>

#include <nn/os/os_Memory.h>
#include <nn/os/os_MemoryTypes.h>
#include <nn/fnd/fnd_ExpHeap.h>

#include <nn/gx.h>
#include <nn/fs.h>

#include "demo.h"

namespace
{
    demo::RenderSystemDrawing s_RenderSystem;
    uptr s_HeapForGx;
    const u32 s_GxHeapSize = 0x400000;
}

typedef struct BitmapFileHeader
{
    u8 bfType[2];
    u16 bfSize[2];
    u16 bfReserved1;
    u16 bfReserved2;
    u16 bfOffBits[2];    
} BitmapFileHeader;

typedef struct BitmapInfoHeader
{
    u32 biSize;
    s32 biWidth;
    s32 biHeight;
    u16 biPlanes;
    u16 biBitCount;
    u32 biCompression;
    u32 biSizeImage;
    s32 biXPixPerMeter;
    s32 biYPixPerMeter;
    u32 biClrUsed;
    u32 biClrImportant;
} BitmapInfoHeader;

// Texture0
GLuint s_BmpTexture0Id = 0;

void Initialize();
void Finalize(void);

void LoadTexture0(nn::y2r::CTR::BlockAlignment y2rAlignment, nn::y2r::CTR::OutputFormat y2rFormat);
void DeleteTexture0(void);

bool DrawFrame(nn::y2r::CTR::BlockAlignment y2rAlignment, nn::y2r::CTR::OutputFormat y2rFormat);
void DrawDisplay0(void);
void DrawDisplay1(void);


// カメラ画像関係
#include <nn/y2r.h>
#include "config.h"
extern u8* g_RgbBuffer;
extern int g_RotatedWidth;
extern int g_RotatedHeight;
extern u8* g_TmpRgbBuffer;
extern u8* g_YuvBuffer[2];

/* そのほか変数 */
int s_Count = 1;


extern nn::fnd::ExpHeap s_AppHeap;
extern int cb_id; //Colorbit id


void Initialize()
{
    //nn::fs::Initialize();
    
    s_AppHeap.Initialize(nn::os::GetDeviceMemoryAddress(), 
        nn::os::GetDeviceMemorySize(), nn::os::ALLOCATE_OPTION_LINEAR);
    s_HeapForGx = reinterpret_cast<uptr>(s_AppHeap.Allocate(s_GxHeapSize));
       
    s_RenderSystem.Initialize(s_HeapForGx, s_GxHeapSize);
        
    // 画像表示用バッファをデバイスメモリから確保
    g_RgbBuffer = reinterpret_cast<u8*>( s_AppHeap.Allocate( WIDTH * HEIGHT * MAX_RGB_BYTES, 64 ) );
	
	// カメラ等に使うバッファもデバイスメモリから確保
	g_TmpRgbBuffer = reinterpret_cast<u8*>( s_AppHeap.Allocate( WIDTH * HEIGHT * MAX_RGB_BYTES, 64));
	g_YuvBuffer[0] = reinterpret_cast<u8*>( s_AppHeap.Allocate( WIDTH * HEIGHT * YUV_BYTES, 64));
	g_YuvBuffer[1] = reinterpret_cast<u8*>( s_AppHeap.Allocate( WIDTH * HEIGHT * YUV_BYTES, 64));

    //LoadTexture0();
}

void Finalize(void)
{
    DeleteTexture0();

    s_RenderSystem.Finalize();

    s_AppHeap.Free(reinterpret_cast<void*>(s_HeapForGx));
	s_AppHeap.Free(g_RgbBuffer);
	s_AppHeap.Free(g_TmpRgbBuffer);
	s_AppHeap.Free(g_YuvBuffer[0]);
	s_AppHeap.Free(g_YuvBuffer[1]);

    s_AppHeap.Finalize();    
}

void LoadTexture0(nn::y2r::CTR::BlockAlignment y2rAlignment, nn::y2r::CTR::OutputFormat y2rFormat)
{
    if ( s_BmpTexture0Id != 0 )
    {
        DeleteTexture0();
    }

    u8* textureDataPtr = g_RgbBuffer;

    GLenum target = GL_TEXTURE_2D;
    GLenum internalFormat = GL_RGB_NATIVE_DMP;
    GLenum format = GL_RGB_NATIVE_DMP;
    if(y2rAlignment == nn::y2r::CTR::BLOCK_8_BY_8)
    {
        target = GL_TEXTURE_2D | NN_GX_MEM_FCRAM | GL_NO_COPY_FCRAM_DMP;
        internalFormat = format = GL_RGB_NATIVE_DMP;
    }
    else
    {
        target = GL_TEXTURE_2D;
        internalFormat = format = GL_RGB;
    }

    GLenum type = GL_UNSIGNED_BYTE;
    switch (y2rFormat)
    {
    case nn::y2r::CTR::OUTPUT_RGB_32:
    case nn::y2r::CTR::OUTPUT_RGB_24:
        type = GL_UNSIGNED_BYTE;
        break;
    case nn::y2r::CTR::OUTPUT_RGB_16_555:
        type = GL_UNSIGNED_SHORT_5_5_5_1;
        break;
    case nn::y2r::CTR::OUTPUT_RGB_16_565:
        type = GL_UNSIGNED_SHORT_5_6_5;
        break;
    }

    GLuint textureId = 0;
    
    s_RenderSystem.GenerateTexture(target,
        internalFormat, g_RotatedWidth, g_RotatedHeight,
        format, type, textureDataPtr, textureId);

    if ( textureId != 0 )
    {
        s_BmpTexture0Id = textureId;
        //NN_LOG("  Create texture (id = %d)\n", textureId);
    }
}

void DeleteTexture0(void)
{        
    if ( s_BmpTexture0Id != 0 )
    {
        bool flag = s_RenderSystem.DeleteTexture(s_BmpTexture0Id);
        if ( flag )
        {
            //NN_LOG("  Delete texture. (id = %d)\n", s_BmpTexture0Id);
            s_BmpTexture0Id = 0;
        }
        else
        {
            NN_TPANIC_("  Failed to delete texture. (id = %d)\n", s_BmpTexture0Id);            
        }
    }
}

bool DrawFrame(nn::y2r::CTR::BlockAlignment y2rAlignment, nn::y2r::CTR::OutputFormat y2rFormat)
{
    LoadTexture0(y2rAlignment, y2rFormat);

    DrawDisplay0();    
    DrawDisplay1();    

    s_Count++;

    s_RenderSystem.WaitVsync(NN_GX_DISPLAY_BOTH);

    return true;
}

void DrawDisplay0(void)
{
    s_RenderSystem.SetClearColor(NN_GX_DISPLAY0, 0.0f, 1.0f, 0.0f, 1.0f);
    s_RenderSystem.SetRenderTarget(NN_GX_DISPLAY0);
    s_RenderSystem.Clear();

    s_RenderSystem.SetColor(0.0f, 1.0f, 0.0f, 0.0f);

    f32 windowCoordinateX = 72.0f;
    f32 windowCoordinateY = 0.0f;
    f32 rectangleWidth = g_RotatedWidth;
    f32 rectangleHeight = g_RotatedHeight;
    s_RenderSystem.FillTexturedRectangle(s_BmpTexture0Id,
        windowCoordinateX, windowCoordinateY,
        rectangleWidth, rectangleHeight,
        g_RotatedWidth, g_RotatedHeight,
        g_RotatedWidth, g_RotatedHeight);

    //Colorbit の読み取り領域描画
    s_RenderSystem.DrawLine(windowCoordinateX+rectangleWidth/4,rectangleHeight/4,windowCoordinateX+rectangleWidth/4*3,rectangleHeight/4);
    s_RenderSystem.DrawLine(windowCoordinateX+rectangleWidth/4*3,rectangleHeight/4,windowCoordinateX+rectangleWidth/4*3,rectangleHeight/4*3);
    s_RenderSystem.DrawLine(windowCoordinateX+rectangleWidth/4*3,rectangleHeight/4*3,windowCoordinateX+rectangleWidth/4,rectangleHeight/4*3);
    s_RenderSystem.DrawLine(windowCoordinateX+rectangleWidth/4,rectangleHeight/4*3,windowCoordinateX+rectangleWidth/4,rectangleHeight/4);

    s_RenderSystem.SwapBuffers();
}

void DrawDisplay1(void)
{
    s_RenderSystem.SetClearColor(NN_GX_DISPLAY1, 0.0f, 0.0f, 1.0f, 0.0f);
    s_RenderSystem.SetRenderTarget(NN_GX_DISPLAY1);
    s_RenderSystem.Clear();
    s_RenderSystem.SetColor( 1.0f, 1.0f, 1.0f );

    glViewport(0, 0, nn::gx::DISPLAY1_WIDTH, nn::gx::DISPLAY1_HEIGHT);
    s_RenderSystem.DrawText(0, 0, "Count: %d", s_Count);
	//Colorbit 読み取り成功時
	if(cb_id >= 0){
    	s_RenderSystem.DrawText(0, 20, "Colorbit: %d", cb_id);
    }

    s_RenderSystem.SwapBuffers();
}
