﻿/*---------------------------------------------------------------------------*
# Colorbit Sample
 *---------------------------------------------------------------------------*/

#include <nn.h>
#include <nn/camera.h>
#include <nn/y2r.h>
#include <nn/hid.h>
#include <nn/applet.h>

#include <string.h>

#include "config.h"
#include "sleep.h"

/*-----------------------------------------------------------------------*
   Colorbit Library Include.
*-----------------------------------------------------------------------*/
#include "cbDecoder.h"


//Colorbit読み取り領域
#define T_ORIGIN_X 64
#define T_ORIGIN_Y 64
#define T_WIDTH 128
#define T_HEIGHT 128

mw::cb::Decoder s_CBDecoder;
nn::fnd::ExpHeap s_AppHeap;
void* CBMalloc(size_t size){return s_AppHeap.Allocate(size);}
void  CBFree(void* ptr) {if(ptr)s_AppHeap.Free(ptr);}
u8 s_ImageBuffer[T_WIDTH*3*T_HEIGHT];	//入力画像データ
int cb_id = -1;


extern int Initialize();
extern void DrawFrame(nn::y2r::CTR::BlockAlignment y2rAlignment, nn::y2r::CTR::OutputFormat y2rFormat);

// Y2R で RGB に変換したデータを受け取るバッファは連続した物理アドレスかつ 4 バイトのアラインが必要。
// ただし、64 バイト未満のアラインの場合は転送速度が落ちる可能性がある。
u8* g_RgbBuffer;


// CTR-SDK7.2.1の修正により、デバイスメモリから確保する必要があります。
//u8 g_TmpRgbBuffer[WIDTH*HEIGHT*MAX_RGB_BYTES] NN_ATTRIBUTE_ALIGN(64);
u8 *g_TmpRgbBuffer;

// カメラのキャプチャデータを受信するバッファは 4 バイトのアラインが必要。
// ただし、64 バイト未満のアラインの場合は転送速度が落ちる可能性がある。
//
// CTR-SDK7.2.1の修正により、デバイスメモリから確保する必要があります。
//u8 g_YuvBuffer[2][WIDTH*HEIGHT*YUV_BYTES] NN_ATTRIBUTE_ALIGN(64);

u8 *g_YuvBuffer[2];

size_t g_CameraOutputSize;
size_t g_TransferUnit;
u32 g_Y2rOutputSize;

int g_RotatedWidth = WIDTH;
int g_RotatedHeight = HEIGHT;

size_t g_RgbBytes;

enum
{
    EVENT_CAMERA_TRANSFER = 0,
    EVENT_CAMERA_ERROR,
    EVENT_CAMERA_NUM
};

typedef struct FrameRate
{
   nn::camera::FrameRate* list;
    char** name;
} FrameRate;


static nn::camera::FrameRate s_FrameRateList[] = {nn::camera::FRAME_RATE_5,
                                                  nn::camera::FRAME_RATE_15,
                                                  nn::camera::FRAME_RATE_20,
                                                  nn::camera::FRAME_RATE_30,
                                                 };
static char* s_FrameRateName[] = {"nn::camera::FRAME_RATE_5",
                                  "nn::camera::FRAME_RATE_15",
                                  "nn::camera::FRAME_RATE_20",
                                  "nn::camera::FRAME_RATE_30",
                                 };

static FrameRate s_FrameRate = {s_FrameRateList, s_FrameRateName};

static int s_ChangeFrameRate = 1;
static bool s_SwitchFrameRateFlag = false;

nn::os::CriticalSection s_CriticalSection;
// カメラのAPIはスレッドセーフでないので、同時に呼ばれる可能性があるものは排他する
nn::os::CriticalSection s_CriticalSectionForStartCapture;


uptr sDeviceMemoryBlockAddress;

void CameraThreadFunc( uptr param )
{
    nn::os::Event  **events  = reinterpret_cast<nn::os::Event**>(param);

    // 最初の転送を設定する
    nn::camera::CTR::SetReceiving(events[EVENT_CAMERA_TRANSFER], reinterpret_cast<void*>(g_YuvBuffer[0]), nn::camera::PORT_CAM1, g_CameraOutputSize, g_TransferUnit);

    u32 yuvWp = 0;

    while(1)
    {
        s32 num = nn::os::WaitObject::WaitAny(reinterpret_cast<nn::os::WaitObject**>(events), EVENT_CAMERA_NUM);

        if(num == EVENT_CAMERA_TRANSFER)
        {
            yuvWp ^= 1;
            nn::camera::CTR::SetReceiving(events[EVENT_CAMERA_TRANSFER], reinterpret_cast<void*>(g_YuvBuffer[yuvWp]), nn::camera::PORT_CAM1, g_CameraOutputSize, g_TransferUnit);

            // このデモではスリープ前のy2rの停止対応をしていないが、カメラのフレーム受信が完了する度に無条件で前回のy2rを停止しているので、停止する問題が発生することはない
            nn::y2r::CTR::StopConversion();
            while(nn::y2r::CTR::IsBusyConversion()) // Enableの間は設定できないので待つ
            {
            }

            // 受信したYUVデータをYUVtoRGBで変換する。
            u32 preYuvWp = yuvWp ^ 1;
            nn::y2r::CTR::SetSendingYuv(reinterpret_cast<void*>(g_YuvBuffer[preYuvWp]), g_CameraOutputSize, WIDTH*YUV_BYTES);
            nn::y2r::CTR::SetReceiving(reinterpret_cast<void*>(g_TmpRgbBuffer), g_Y2rOutputSize, WIDTH*8*g_RgbBytes);

            // スリープ中の場合はエラーが返るが、このデモでは変換できた絵を出すことしかしていないので、特に対応は必要ない
            nn::y2r::CTR::StartConversion();
        }
        else if(num == EVENT_CAMERA_ERROR)
        {
            NN_LOG("camera_sample: error or reboot occured\n");

            // キャプチャ再開
            nn::camera::CTR::SetReceiving(events[EVENT_CAMERA_TRANSFER], reinterpret_cast<void*>(g_YuvBuffer[yuvWp]), nn::camera::PORT_CAM1, g_CameraOutputSize, g_TransferUnit);

            s_CriticalSectionForStartCapture.Enter();
            nn::camera::CTR::StartCapture(nn::camera::PORT_CAM1);
            s_CriticalSectionForStartCapture.Leave();
        }
    }
}

void CameraVsyncThreadFunc( uptr param )
{
    nn::os::Event  *vsyncEvent  = reinterpret_cast<nn::os::Event*>(param);

    while(1)
    {
        vsyncEvent->Wait();

        if(s_SwitchFrameRateFlag)
        {
            if(s_CriticalSection.TryEnter())
            {
                nn::camera::StopCapture(nn::camera::PORT_CAM1);
                nn::Result result = nn::camera::SetFrameRate(nn::camera::CTR::SELECT_OUT1, s_FrameRate.list[s_ChangeFrameRate]);
                if(result == nn::camera::CTR::ResultFatalError())
                {
                    NN_PANIC("Camera has broken.\n"); 
                }
                else if(result != nn::camera::CTR::ResultIsSleeping())
                {
                    // 蓋閉じ中によるカメラ設定が失敗した場合は、後ほどやり直す
                    s_SwitchFrameRateFlag = false;
                }
                s_CriticalSectionForStartCapture.Enter();
                nn::camera::StartCapture(nn::camera::PORT_CAM1);
                s_CriticalSectionForStartCapture.Leave();
                s_CriticalSection.Leave();
            }
        }
    }
}

void CheckSleepRequest( void )
{
    if ( SleepHandler::IsSleepRequested() )
    {
        SleepHandler::SleepSystem();

        // 再開後の処理
        nn::gx::StartLcdDisplay();
    }
}

bool CheckCameraResult( nn::Result result )
{
    if(result == nn::camera::CTR::ResultIsSleeping())
    {
        // 蓋閉じ中ならリトライが必要
        return false;
    }
    if(result == nn::camera::CTR::ResultFatalError())
    {
        // FATALエラーは、それぞれのアプリにとって必要な対応を行う
        NN_PANIC("Camera has broken.\n"); 
    }
    return true;
}

extern "C" void nnMain( void )
{
    nn::Result result;


    // applet の初期化
    SleepHandler::Initialize();

    s_CriticalSection.Initialize();
    s_CriticalSectionForStartCapture.Initialize();

	// 画像表示用の初期化
    if(Initialize() < 0)
    {
        NN_PANIC("failed gx init\n");
    }

    // CAMERA 初期化
    result = nn::camera::CTR::Initialize();
    // このデモはカメラが動作しないと意味がないので、蓋が開けられるまで初期化をリトライする
    // ただし、本体スリープ判定はしておく必要がある
    while(!CheckCameraResult(result))
    {
        CheckSleepRequest();
        nn::os::Thread::Sleep(nn::fnd::TimeSpan::FromMilliSeconds(100));
        result = nn::camera::CTR::Initialize();
    }
    // Initialize関数については、カメラ設定関数が返さないエラーについても対応が必要
    if(result == nn::camera::CTR::ResultUsingOtherProcess())
    {
        // 裏で動いているアプレットがカメラ使用中のままであることはあり得ない
        // このエラーが返ってくる場合は、そのアプレットのバグである
        NN_PANIC("Running illegal applet.\n");
    }

    // Y2R 初期化
    nn::y2r::CTR::Initialize();

    // カメラの設定
    nn::os::Event errorIntr, vsyncIntr;

    // カメラ設定の中で一括設定可能な項目が2つ以上あれば、一括設定関数を使ったほうがよい
    // コンテキスト指定ありの項目は、今回画像サイズしか変更しかしないので、一括設定関数は使用しない
    // コンテキスト指定なしの項目は、今回フレームレートの変更しかしないので、一括設定関数は使用しない

    // バッファにためるライン数を指定
    s16 transferLine = nn::camera::CTR::GetMaxLines( WIDTH, HEIGHT );
    if(WIDTH < 16) // GetMaxLinesは最大ライン数を取得する関数なので、元々のラインがそれ以下の場合もあり得る。
    {
        transferLine = HEIGHT;
    }
    nn::camera::CTR::SetTransferLines(nn::camera::PORT_CAM1, transferLine, WIDTH, HEIGHT);

    // 割り込みハンドルの取得
    nn::camera::CTR::GetBufferErrorInterruptEvent(&errorIntr, nn::camera::PORT_CAM1);
    nn::camera::CTR::GetVsyncInterruptEvent(&vsyncIntr, nn::camera::PORT_CAM1);

    // フレームレートの変更
    result = nn::camera::CTR::SetFrameRate(nn::camera::CTR::SELECT_IN1_OUT1, nn::camera::CTR::FRAME_RATE_15);
    while(!CheckCameraResult(result))
    {
        CheckSleepRequest();
        nn::os::Thread::Sleep(nn::fnd::TimeSpan::FromMilliSeconds(100));
        result = nn::camera::CTR::SetFrameRate(nn::camera::CTR::SELECT_IN1_OUT1, nn::camera::CTR::FRAME_RATE_15);
    }

    // 画像サイズの変更
    result = nn::camera::CTR::SetSize(nn::camera::CTR::SELECT_IN1_OUT1, nn::camera::CTR::SIZE_VGA, nn::camera::CTR::CONTEXT_A);
    while(!CheckCameraResult(result))
    {
        CheckSleepRequest();
        nn::os::Thread::Sleep(nn::fnd::TimeSpan::FromMilliSeconds(100));
        result = nn::camera::CTR::SetSize(nn::camera::CTR::SELECT_IN1_OUT1, nn::camera::CTR::SIZE_VGA, nn::camera::CTR::CONTEXT_A);
    }

    // カメラを起動状態にする
    result = nn::camera::CTR::Activate(nn::camera::CTR::SELECT_OUT1);
    while(!CheckCameraResult(result))
    {
        CheckSleepRequest();
        nn::os::Thread::Sleep(nn::fnd::TimeSpan::FromMilliSeconds(100));
        result = nn::camera::CTR::Activate(nn::camera::CTR::SELECT_OUT1);
    }

    // トリミングの設定
    nn::camera::CTR::SetTrimming(nn::camera::PORT_CAM1, true);
    nn::camera::CTR::SetTrimmingParamsCenter(nn::camera::PORT_CAM1, WIDTH, HEIGHT, CAMERA_WIDTH, CAMERA_HEIGHT);


    // Y2Rの設定
    nn::os::Event transEndIntr;

    // 変換中なら強制停止する。
    nn::y2r::CTR::StopConversion();
    while(nn::y2r::CTR::IsBusyConversion()) // Enableの間は設定できないので待つ
    {
    }

    // Y2R の設定は一括関数で行なったほうが早い
    nn::y2r::CTR::PackageParameter y2rPackageParameter;
    // 入力フォーマットの変更
    y2rPackageParameter.inputFormat = nn::y2r::CTR::INPUT_YUV422_BATCH;
    // 出力フォーマットの変更
    nn::y2r::CTR::OutputFormat format = nn::y2r::CTR::OUTPUT_RGB_24;
    y2rPackageParameter.outputFormat = format;
    // 出力データの回転角度の変更
    nn::y2r::CTR::Rotation rotation = nn::y2r::CTR::ROTATION_NONE;
    y2rPackageParameter.rotation = rotation;
    // 出力データの並び順の変更
    
    nn::y2r::CTR::BlockAlignment alignment = nn::y2r::CTR::BLOCK_LINE;
    //Colorbitの読み取りに使用するためBLOCK_LINEフォーマットを使用

    y2rPackageParameter.blockAlignment = alignment;
    // 入力データのサイズを指定
    y2rPackageParameter.inputLineWidth = WIDTH;
    y2rPackageParameter.inputLines = HEIGHT;
    // 変換係数の設定
    nn::y2r::CTR::StandardCoefficient no;
    result = nn::camera::CTR::GetSuitableY2rStandardCoefficient(&no);
    while(!CheckCameraResult(result))
    {
        CheckSleepRequest();
        nn::os::Thread::Sleep(nn::fnd::TimeSpan::FromMilliSeconds(100));
        result = nn::camera::CTR::GetSuitableY2rStandardCoefficient(&no);
    }
    y2rPackageParameter.standardCoefficientNo = no;
    // アルファ値の設定(このデモでは使用しない)
    y2rPackageParameter.alpha = 0xFF;
    nn::y2r::CTR::SetPackageParameter(y2rPackageParameter);

    g_RgbBytes = nn::y2r::CTR::GetOutputFormatBytes(nn::y2r::CTR::OUTPUT_RGB_24);

    // 割り込みハンドルの取得
    nn::y2r::CTR::GetTransferEndEvent(&transEndIntr);

    g_TransferUnit = nn::camera::CTR::GetTransferBytes(nn::camera::PORT_CAM1);

    g_CameraOutputSize = nn::camera::CTR::GetFrameBytes(WIDTH, HEIGHT);
    g_Y2rOutputSize = nn::y2r::CTR::GetOutputImageSize(WIDTH, HEIGHT, nn::y2r::CTR::OUTPUT_RGB_24);

    // hid の初期化
    result = nn::hid::Initialize();
    NN_UTIL_PANIC_IF_FAILED(result);

    nn::os::Event dataTransferIntr;

    // カメラのエラー/データ転送処理用スレッド作成
    nn::os::Thread cameraThread;
    nn::os::Event *events[EVENT_CAMERA_NUM] = {&dataTransferIntr, &errorIntr};
    cameraThread.StartUsingAutoStack(
        CameraThreadFunc,
        reinterpret_cast<uptr>(events),
        4096,
        nn::os::Thread::GetCurrentPriority()-2 );   // メインスレッドより優先度は高い

    // カメラのVSYNC処理用スレッド作成
    nn::os::Thread cameraVsyncThread;
    cameraVsyncThread.StartUsingAutoStack(
        CameraVsyncThreadFunc,
        reinterpret_cast<uptr>(&vsyncIntr),
        4096,
        nn::os::Thread::GetCurrentPriority()-1 );   // メインスレッドより優先度は高い

	//BLOCK_LINEフォーマットを使用しているため上下反転
    result = nn::camera::FlipImage(nn::camera::CTR::SELECT_IN1_OUT1, nn::camera::FLIP_VERTICAL);
    while(!CheckCameraResult(result))
    {
        CheckSleepRequest();
        nn::os::Thread::Sleep(nn::fnd::TimeSpan::FromMilliSeconds(100));
        result = nn::camera::FlipImage(nn::camera::CTR::SELECT_IN1_OUT1, nn::camera::FLIP_VERTICAL);
    }


    //Colorbit初期化
    s_CBDecoder.Initialize(CBMalloc,CBFree);
    s_CBDecoder.SetParams(mw::cb::DECODER_NONUSE_IMAGE_CORRECTION, mw::cb::CODING_LD5_C15_CORRECTION_LEVEL_2);
    //入力画像が上下反転しているためY方向のみ反転した座標をセット
	int location_table[15][2] = {
    	{34,T_HEIGHT-30},{56,T_HEIGHT-30},{76,T_HEIGHT-30},{98,T_HEIGHT-30},{120,T_HEIGHT-30},
	{24,T_HEIGHT-64},{44,T_HEIGHT-62},{66,T_HEIGHT-62},{88,T_HEIGHT-62},{108,T_HEIGHT-62},
	{12,T_HEIGHT-98},{34,T_HEIGHT-98},{56,T_HEIGHT-98},{76,T_HEIGHT-98},{98,T_HEIGHT-98}
    };
    s_CBDecoder.SetColorLocation( location_table);

    // キャプチャ開始
    nn::camera::CTR::ClearBuffer(nn::camera::PORT_CAM1);
    nn::camera::CTR::StartCapture(nn::camera::PORT_CAM1);

    int frameCount = 0;
    bool changed = false;
/*
    // y2r設定値の一括取得関数の動作テスト
    {
        nn::y2r::CTR::PackageParameter getY2rPackageParameter;
        nn::y2r::CTR::GetPackageParameter(&getY2rPackageParameter);
        nn::dbg::detail::Printf("[Y2R param] InputFormat = %d\n", getY2rPackageParameter.inputFormat.Get());
        nn::dbg::detail::Printf("[Y2R param] OutputFormat = %d\n", getY2rPackageParameter.outputFormat.Get());
        nn::dbg::detail::Printf("[Y2R param] Rotation = %d\n", getY2rPackageParameter.rotation.Get());
        nn::dbg::detail::Printf("[Y2R param] BlockAlignment = %d\n", getY2rPackageParameter.blockAlignment.Get());
        nn::dbg::detail::Printf("[Y2R param] InputLineWidth = %d\n", getY2rPackageParameter.inputLineWidth);
        nn::dbg::detail::Printf("[Y2R param] InputLines = %d\n", getY2rPackageParameter.inputLines);
        nn::dbg::detail::Printf("[Y2R param] StandardCoefficientNo = %d\n", getY2rPackageParameter.standardCoefficientNo.Get());
        nn::dbg::detail::Printf("[Y2R param] Alpha = %d\n", getY2rPackageParameter.alpha);
    }
*/
    while(1)
    {
        // *** 注意 ***
        //
        // Y2R にはハードウェア不具合があり、カメラと Y2R を同時に使用したとき、
        // カメラのバッファエラーの発生時の復帰処理によって、
        // タイミング依存で Y2R 転送がハングアップする可能性があります。
        // そのため、nn::y2r::GetTransferEndEvent で取得される変換完了イベントが
        // 永遠に Signal されない可能性があります。
        // 発生条件と対策の詳細については、y2r の関数リファレンスマニュアルをご参照ください。
        // 
        // 本サンプルでは、上記のイベントを Wait(0) でポーリングしており、
        // カメラ画像が取得されるたびに Y2R 変換を強制的に終了して
        // 新たな変換を開始する実装になっているので、不具合は発生しません。
        //
        // *** 注意おわり ***

        if(transEndIntr.Wait(0))
        {
            if(!changed)
            {
                // 出力画像の回転への対応
                if(rotation == nn::y2r::CTR::ROTATION_CLOCKWISE_90)
                {
                    g_RotatedWidth = HEIGHT;
                    g_RotatedHeight = WIDTH;
                    if(alignment == nn::y2r::CTR::BLOCK_LINE)
                    {
                        int block, unit;
                        for(block = 0; block < HEIGHT/8; block++)
                        {
                            for(unit = 0; unit < 8*WIDTH/8; unit++)
                            {
                                int src_offset = block*8*WIDTH*g_RgbBytes + unit*8*g_RgbBytes;
                                int dst_offset = (g_RotatedWidth-8*(block+1))*g_RgbBytes + unit*g_RotatedWidth*g_RgbBytes;
                                memcpy(&g_RgbBuffer[dst_offset], &g_TmpRgbBuffer[src_offset], 8*g_RgbBytes);
                            }
                        }
                    }
                    else if(alignment == nn::y2r::CTR::BLOCK_8_BY_8)
                    {
                        int block, unit;
                        for(block = 0; block < HEIGHT/8; block++)
                        {
                            for(unit = 0; unit < WIDTH/8; unit++)
                            {
                                int src_offset = block*8*WIDTH*g_RgbBytes + unit*8*8*g_RgbBytes;
                                int dst_offset = (g_RotatedWidth-8*(block+1))*8*g_RgbBytes + unit*8*g_RotatedWidth*g_RgbBytes;
                                memcpy(&g_RgbBuffer[dst_offset], &g_TmpRgbBuffer[src_offset], 8*8*g_RgbBytes);
                            }
                        }
                    }
                }
                if(rotation == nn::y2r::CTR::ROTATION_CLOCKWISE_180)
                {
                    g_RotatedWidth = WIDTH;
                    g_RotatedHeight = HEIGHT;

                    int block;
                    for(block = 0; block < HEIGHT/8; block++)
                    {
                        int src_offset = block*WIDTH*8*g_RgbBytes;
                        int dst_offset = (g_RotatedHeight/8 -1-block)*g_RotatedWidth*8*g_RgbBytes;
                        memcpy(&g_RgbBuffer[dst_offset], &g_TmpRgbBuffer[src_offset], 8*WIDTH*g_RgbBytes);
                    }
                }
                if(rotation == nn::y2r::CTR::ROTATION_CLOCKWISE_270)
                {
                    g_RotatedWidth = HEIGHT;
                    g_RotatedHeight = WIDTH;
                    if(alignment == nn::y2r::CTR::BLOCK_LINE)
                    {
                        int block, unit;
                        for(block = 0; block < HEIGHT/8; block++)
                        {
                            for(unit = 0; unit < 8*WIDTH/8; unit++)
                            {
                                int src_offset = block*8*WIDTH*g_RgbBytes + unit*8*g_RgbBytes;
                                int dst_offset = block*8*g_RgbBytes + unit*g_RotatedWidth*g_RgbBytes;
                                memcpy(&g_RgbBuffer[dst_offset], &g_TmpRgbBuffer[src_offset], 8*g_RgbBytes);
                            }
                        }
                    }
                    else if(alignment == nn::y2r::CTR::BLOCK_8_BY_8)
                    {
                        int block, unit;
                        for(block = 0; block < HEIGHT/8; block++)
                        {
                            for(unit = 0; unit < WIDTH/8; unit++)
                            {
                                int src_offset = block*8*WIDTH*g_RgbBytes + unit*8*8*g_RgbBytes;
                                int dst_offset = block*8*8*g_RgbBytes + unit*8*g_RotatedWidth*g_RgbBytes;
                                memcpy(&g_RgbBuffer[dst_offset], &g_TmpRgbBuffer[src_offset], 8*8*g_RgbBytes);
                            }
                        }
                    }
                }
                if(rotation == nn::y2r::CTR::ROTATION_NONE)
                {
                    g_RotatedWidth = WIDTH;
                    g_RotatedHeight = HEIGHT;
                    memcpy(g_RgbBuffer, g_TmpRgbBuffer, g_RotatedWidth*g_RotatedHeight*g_RgbBytes);
                }


                if(alignment == nn::y2r::BLOCK_LINE)
                {
                    char tmp;
                    for(int i = 0; i < WIDTH*HEIGHT*g_RgbBytes; i+=g_RgbBytes)
                    {
                        tmp = g_RgbBuffer[i];
                        g_RgbBuffer[i] = g_RgbBuffer[i+g_RgbBytes-1];
                        g_RgbBuffer[i+g_RgbBytes-1] = tmp;
                        tmp = g_RgbBuffer[i+1];
                        g_RgbBuffer[i+1] = g_RgbBuffer[i+g_RgbBytes-2];
                        g_RgbBuffer[i+g_RgbBytes-2] = tmp;
                    }
                }

		
        //入力画像データ コピー
		int widthStep = WIDTH*3;
		int widthCount = 0;
		for(int i = T_ORIGIN_Y; i < T_ORIGIN_Y + T_HEIGHT; i++,widthCount++){
			memcpy(&s_ImageBuffer[T_WIDTH*3*widthCount],&g_RgbBuffer[widthStep*i + T_ORIGIN_X*g_RgbBytes],T_WIDTH*g_RgbBytes);
		}

        //色抽出点確認のため白色にセット
		for(int i = 0; i < 15; i++){
			int x = location_table[i][0]+T_ORIGIN_X;
			int y = location_table[i][1]+T_ORIGIN_Y;
			g_RgbBuffer[widthStep*y+x*3+0] = 255;
			g_RgbBuffer[widthStep*y+x*3+1] = 255;
			g_RgbBuffer[widthStep*y+x*3+2] = 255;
		} 

        //Colorbit 読み取り
        unsigned long long colorbit_id;
		unsigned char color_seq[15];
		int error = s_CBDecoder.QueryImage((void*)s_ImageBuffer,T_WIDTH,T_HEIGHT,&colorbit_id,color_seq);
		if(error < 0){
			//NN_LOG("Colorbit Decode Error : %d\n",error);
			cb_id = -1;
		}else{
			cb_id = (int)colorbit_id;
		}

        //NN_LOG("camera_sample: frameCount = %d\n", frameCount);
		frameCount++;
                DrawFrame(alignment, format); // 画像表示
            }
            else
            {
                changed = false;
            }
        }

        // 本体スリープ判定
        if ( SleepHandler::IsSleepRequested() )
        {
            SleepHandler::SleepSystem();

            // 再開後の処理
            nn::gx::StartLcdDisplay();
        }

        // HOMEボタン処理
        if ( nn::applet::IsExpectedToProcessHomeButton() )
        {
            nn::applet::ProcessHomeButton();
            AppletWakeupState r = nn::applet::WaitForStarting();
            NN_UNUSED_VAR(r);

            // 待ちうけからの戻り終了チェック
            if ( nn::applet::IsExpectedToCloseApplication() )
            {
                break;
            }

            // GPU レジスタ設定の復帰
            nngxUpdateState(NN_GX_STATE_ALL);
            nngxValidateState(NN_GX_STATE_ALL,GL_TRUE);
        }

        // 電源ボタン処理
        if ( nn::applet::IsExpectedToProcessPowerButton() )
        {
            nn::applet::ProcessPowerButton();
            AppletWakeupState r = nn::applet::WaitForStarting();
            NN_UNUSED_VAR(r);

            // 待ちうけからの戻り終了チェック
            if ( nn::applet::IsExpectedToCloseApplication() )
            {
                break;
            }

            // GPU レジスタ設定の復帰
            nngxUpdateState(NN_GX_STATE_ALL);
            nngxValidateState(NN_GX_STATE_ALL,GL_TRUE);
        }

        // 終了要求
        if ( nn::applet::IsExpectedToCloseApplication() )
        {
            break;
        }
    }

    // スリープハンドラの終了処理
    SleepHandler::Finalize();

    //Colorbitの終了処理
    s_CBDecoder.Finalize();

    nn::applet::CloseApplication();

    // nn::applet::CloseApplication() からは戻りません。
}
