﻿/*---------------------------------------------------------------------------*
# Colorbit Sample
 *---------------------------------------------------------------------------*/

#include <nn/applet.h>

#include "sleep.h"

/*------------------------------------------------------------------------*
        本体スリープに関連する処理
 *------------------------------------------------------------------------*/
nn::os::LightEvent  SleepHandler::s_AwakeEvent;
bool                SleepHandler::s_IsInitialized = false;

/*------------------------------------------------------------------------*/
void SleepHandler::Initialize( void )
{
    s_AwakeEvent.Initialize(false);
    nn::applet::SetSleepQueryCallback(SleepQueryCallback, 0);
    nn::applet::SetAwakeCallback(AwakeCallback, 0);
    nn::applet::Enable();
    s_IsInitialized = true;
}

/*------------------------------------------------------------------------*/
void SleepHandler::Finalize( void )
{
    nn::applet::DisableSleep();
    s_AwakeEvent.Finalize();
    nn::applet::SetSleepQueryCallback(NULL, 0);
    nn::applet::SetAwakeCallback(NULL, 0);
    s_IsInitialized = false;
}

/*------------------------------------------------------------------------*
    システムスリープの指示が来ているかどうかを返します。
 *------------------------------------------------------------------------*/
bool SleepHandler::IsSleepRequested( void )
{
    NN_TASSERT_(s_IsInitialized);
    
    if ( nn::applet::IsExpectedToReplySleepQuery() )
        return true;
    
    return false;
}

/*------------------------------------------------------------------------*
    本体スリープ状態に移行させるための処理を行います。
    グラフィックスなどの処理がスリープに入っても問題ない
    適切なタイミングで呼び出す必要があります。
 *------------------------------------------------------------------------*/
void SleepHandler::SleepSystem( void )
{
    // 指示が来ていない場合は何もせず戻る。
    if ( !IsSleepRequested() )
    {
        return;
    }
    
    nn::applet::ReplySleepQuery(nn::applet::REPLY_ACCEPT);
    
    s_AwakeEvent.Wait();
}

/*------------------------------------------------------------------------*
    スリープ問い合わせ時に呼ばれるコールバック
 *------------------------------------------------------------------------*/
AppletQueryReply SleepHandler::SleepQueryCallback( uptr arg )
{
    NN_UNUSED_VAR(arg);
    
    s_AwakeEvent.ClearSignal();
    
    if ( !nn::applet::IsActive() )
    {
        return nn::applet::REPLY_ACCEPT;
    }

    if ( s_IsInitialized )
    {
        return nn::applet::REPLY_LATER;
    }
    else
    {
        return nn::applet::REPLY_REJECT;
    }
}

/*------------------------------------------------------------------------*
    スリープ復帰時に呼ばれるコールバック
 *------------------------------------------------------------------------*/
void SleepHandler::AwakeCallback( uptr arg )
{
    NN_UNUSED_VAR(arg);
    s_AwakeEvent.Signal();
}

