﻿/*---------------------------------------------------------------------------*
  Project:  Horizon
  File:     GuiLabel.cpp

  Copyright (C)2009-2011 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  $Rev: 42832 $
 *---------------------------------------------------------------------------*/

#include "GuiLabel.h"

namespace scene
{

// コンストラクタです。
Label::Label(u32 id, s32 x, s32 y, u32 width, u32 height, const wchar_t* pText, void* pExtraData, f32 fontScale)
    : ControlBase(CONTROL_TYPE_LABEL, id, x, y, width, height, pText, pExtraData, fontScale)
{
    m_textAlign   = TEXT_ALIGN_LEFT;
    m_borderWidth = 0.0f;
    m_state       = CONTROL_STATE_DEFAULT;

    // 文字の色
    m_textColors[CONTROL_STATE_DEFAULT]  = nn::util::Color8(192, 192, 192, 255);
    m_textColors[CONTROL_STATE_DISABLED] = nn::util::Color8( 32,  32,  32, 255);

    // 枠線の色
    m_borderColors[CONTROL_STATE_DEFAULT]   = nn::util::Color8(0, 0, 0, 0);
    m_borderColors[CONTROL_STATE_DISABLED]  = nn::util::Color8(0, 0, 0, 0);
}

// デストラクタです。
Label::~Label()
{
}

// ペンで押された時の処理
void Label::OnPenDown()
{
    // 何もしない
}

// ペンが離された時の処理
void Label::OnPenUp(bool isIn)
{
    (void)isIn;
}

// ペンでスライドされた時の処理
void Label::OnPenSlide(bool isIn)
{
    (void)isIn;
}

// ペンでタッチされた時の処理
void Label::OnPenTouch()
{
}

// 描画時の処理
void Label::OnDraw()
{
    // レンダラ
    demo::RenderSystemDrawing* pRenderSystem = scene::GetRenderSystem();
    // テキストライター
    TextWriter* pTextWriter = scene::GetTextWriter();

    // 枠線
    if (m_borderWidth >= 0.0f)
    {
        nn::util::FloatColor color = m_borderColors[m_state];

        // 箱を描画する
        pRenderSystem->SetColor(color.r, color.g, color.b, color.a);
        scene::util::DrawBox(m_x, m_y, m_width, m_height, m_borderWidth);
    }

    // テキスト
    if (m_textLength > 0)
    {
        nn::util::Color8 color = m_textColors[m_state];
        // スケール設定
        pTextWriter->SetFontScale(m_TextScale);

        // テキスト描画位置
        s32 tx = CalculateTextX(2, 0);
        s32 ty = CalculateTextY(0);

        // テキストを描画する
        pTextWriter->SetTextColor(color.r, color.g, color.b);
        pTextWriter->PutString(tx, ty, m_pText);
    }
}

}
